/*
**  ORDCRIT.g
**
**  purpose : calculation of criteria for ARX order selection
**
**  format  :  { AIC,p_AIC,SIC,p_SIC,LB,p_LB,pval_LB,BP,p_BP,pval_BP} = ORDCRIT(y,x,p_max,n_corr,sig_l);
**
**  input   : y     : (Txk) matrix, containing time series
**            x     : (Txm) matrix, containg exogenous series (will not be
**                    lagged in proc OR 0 to indicate that no exogenous
**                    variables are present
**            p_max : scalar, maximum AR order
**            n_corr: scalar, number of residual correlations to use 
**                    in Ljung-Box test statistic
**            sig_l : scalar, significance level to use in LB test
**
**  output  : AIC   : (p_max x 1) vector, containing values of Akaike
**                    Information Criterion, cf. (4.3.2) in Lutkepohl (1991)
**            p_AIC : scalar, AR order for which AIC is minimized
**            SC    : (p_max x 1) vector, containing values of Schwarz
**                    Criterion, cf. (4.3.9) in Lutkepohl (1991)
**            p_SC  : scalar, AR order for which SC is minimized
**            LB    : (p_max x n_corr) matrix, containing values of LB test
**                    statistic. (i,j)-th element contains statistic
**                    using first j autocorrelations for residuals from
**                    ARX(i) model
**            p_LB  : (n_corr x 1), smallest AR order for which LB is 
**                    not significant at sig_l
**            BP    : Breusch-Pagan test of autocorrelation 
**                    its p-value
**
**  globals :
**
**  remarks :
**
**  authors : Dick van Dijk and Stefan Lundbergh
**
**  date    : First attempt on 13 October 1999
**            Last revision on 13 October 1999
**
*/
proc (10) = ORDCRIT(y,x,p_max,n_corr,sig_l);
  local AIC,SIC,LB,pval_LB,BP,pval_BP,T,k_x,_dat,_y,_x,i,phi_l,_y_fit,_e_fit,tx,corr,Tk,
        p_AIC,p_SIC,p_LB,p_BP,j,t0,t1,t2,t3;

AIC=zeros(p_max+1,1);
SIC=zeros(p_max+1,1);
LB=zeros(p_max+1,n_corr);
pval_LB=zeros(p_max+1,n_corr);
BP=zeros(p_max+1,n_corr);
pval_BP=zeros(p_max+1,n_corr);

T=rows(y)-p_max;
k_x=cols(x);
_dat=y[p_max+1:T+p_max]~x[p_max+1:T+p_max,.]~
     trimr(lagn(y,seqa(1,1,p_max)),p_max,0);
_y=_dat[.,1];

i=-1;
do until (i==p_max);
  i=i+1;
  _x=_dat[.,2:2+k_x+i-1];

  /* Estimate ARX(p) model */
  phi_l=invpd(_x'_x)*(_x'_y);

  /* Compute output, residuals, AIC, SIC, residual correlations and LB statistic*/
  _y_fit=_x*phi_l;
  _e_fit=_y-_y_fit;
  AIC[i+1]=ln(meanc(_e_fit.^2))+2*(i+k_x)./T;
  SIC[i+1]=ln(meanc(_e_fit.^2))+(i+k_x)*ln(T)./T;

  tx=missrv(lagn(_e_fit,seqa(0,1,n_corr+1)),0);
  corr=((tx[.,1]'tx[.,2:n_corr+1])./(tx[.,1]'tx[.,1]))';

  Tk=seqa(T-1,-1,n_corr);
  LB[i+1,.]=T*(T+2)*cumsumc((corr.^2)./Tk)';
  /* NOTE: degrees of freedom is corrected for number of estimated parameters in ARX model */
  if (i+k_x .LE n_corr);
    pval_LB[i+1,i+k_x:n_corr]=CDFCHIC(LB[i+1,i+k_x:n_corr],seqa(1,1,n_corr-i-k_x+1)');
  endif;

  j=0;
  do until (j==n_corr);
    j=j+1;
    BP[i+1,j]=T*(tx[.,1]'(_x~tx[.,2:j+1]))*
                    invpd((_x~tx[.,2:j+1])'(_x~tx[.,2:j+1]))*
                   ((_x~tx[.,2:j+1])'tx[.,1])./(tx[.,1]'tx[.,1]); 

    pval_BP[i+1,j]=CDFCHIC(BP[i+1,j],j);
  endo;

endo;

p_AIC=minindc(AIC)-1;
p_SIC=minindc(SIC)-1;
p_LB=zeros(n_corr,1);
p_BP=zeros(n_corr,1);
i=0;
do until (i==n_corr);
  i=i+1; 
  if (sumc(pval_LB[.,i] .GT sig_l)==0); 
    p_LB[i]=p_max;
  else;
    p_LB[i]=maxindc(pval_LB[.,i] .GT sig_l) - 1;
  endif;
  if (sumc(pval_BP[.,i] .GT sig_l)==0); 
    p_BP[i]=p_max;
  else;
    p_BP[i]=maxindc(pval_BP[.,i] .GT sig_l) - 1;
  endif;
endo;

retp(AIC,p_AIC,SIC,p_SIC,LB,p_LB,pval_LB,BP,p_BP,pval_BP);
endp;
